#!/usr/bin/env python

# # Example: Using QTrack to tracking AEWs in reanalysis data
#
# This example will demonstrate the basic functions of AEW tracking from the Qtrack module. In order to track AEWs, we must proceed through a few steps:
#
# 1. Download or location data for tracking. Data requirments: 700hPa wind (u and v component), 6 hourly timesteps, 1 degree by 1 degree resolution.
# 2. `prep_data` function to prep the input data for tracking.
# 4. `curvvort` Compute the curvature vorticity from the wind fields, and then take radial averages at each gridpoint to smooth it.
# 5. `tracking` Run the AEW tracking code.
# 6. `postproc` Run postprocessing on AEW data, including the creation of a netCDF4 file.


import time

import qtrack
from qtrack.curvvort import compute_curvvort
from qtrack.tracking import run_postprocessing, run_tracking

tstart = time.time()


# ### NAME FILES
prepped_data_save = "adjusted_data.nc"
curv_file_out = "curv_vort_era5_test.nc"
AEW_raw_save_file = "AEW_tracks_raw.nc"
AEW_final_nc_file = "AEW_tracks_post_processed.nc"
AEW_final_obj_file = "AEW_tracks_post_processed.pkl"
year_in = 2021

# ### DOWNLOAD EXAMPLE DATA (OPTIONAL)
qtrack.download_examples("mpas_2021092400", "")

# ### Prep data

qtrack.prep_data(data_in="mpas_30km_run_2021092400.nc", data_out=prepped_data_save, cut_lev_val=70000)

# ### Curvature vorticity calculation
data_file_in = prepped_data_save  # "prepped_data_for_tracking.nc"#"analysis_and_forecast_GFS_2024062612.nc"
compute_curvvort(data_file_in, curv_file_out, njobs_in=-1)

# ### AEW Tracking step
run_tracking(input_file=curv_file_out, save_file=AEW_raw_save_file)


# ### AEW Postprocessing step
run_postprocessing(input_file=AEW_raw_save_file, real_year_used=year_in, curv_data_file=curv_file_out, save_obj_file=AEW_final_obj_file, save_nc_file=AEW_final_nc_file)

# ### OUTPUT TIME
tend = time.time()
elapsed_time = tstart - tend
print("Time to run computation: " + str(round(tend - tstart, 2)) + " seconds | " + str(round((tend - tstart) / 60, 2)) + " minutes")
print()



"""
QTrack - African Easterly Wave Tracking in Model and Reanalysis Data
"""

__version__ = "0.0.4"

from .core import COMPUTE_CURV_VORT_NON_DIV_UPDATE, download_examples, prep_data

__all__ = [
    "COMPUTE_CURV_VORT_NON_DIV_UPDATE",
    "download_examples",
    "prep_data",
    #
    "season",
    "AEW",
    "AEW_CCKW",
]


class season:
    def __init__(self, year, AEW_group):
        self.year = year
        self.AEW_group = AEW_group

    def number_of_waves(self):
        return len(self.AEW_group)

    def get_wave(self, wavenumber):
        return self.AEW_group[wavenumber - 1]

    def waves_with_TC(self):
        waves_TC = []
        for i in range(len(self.AEW_group)):
            TC_ans = self.AEW_group[i].connected_TC
            if TC_ans:
                waves_TC.append(i + 1)
        return waves_TC


class AEW:
    def __init__(
        self,
        year,
        number,
        time,
        lon,
        lat,
        smooth_lon,
        smooth_lat,
        strength,
        over_africa,
        connected_TC,
        connected_TC_name="N/A",
        genesis_time="N/A",
    ):
        self.year = year
        self.number = number
        self.time = time
        self.lon = lon
        self.lat = lat
        self.strength = strength
        self.over_africa = over_africa
        self.connected_TC = connected_TC
        self.connected_TC_name = connected_TC_name
        self.TC_genesis_time = genesis_time
        self.smooth_lon = smooth_lon
        self.smooth_lat = smooth_lat

    def get_data(self):
        return self.time, self.lon, self.lat


class AEW_CCKW:
    def __init__(
        self,
        year,
        number,
        time,
        lon,
        lat,
        smooth_lon,
        smooth_lat,
        strength,
        over_africa,
        connected_TC,
        TB_lat_range,
        VP_lat_range,
        CCKW_TB,
        CCKW_VP,
        sig_TB_peak,
        sig_VP_peak,
        sig_TB_valley,
        sig_VP_valley,
        connected_TC_name="N/A",
        genesis_time="N/A",
    ):
        self.year = year
        self.number = number
        self.time = time
        self.lon = lon
        self.lat = lat
        self.strength = strength
        self.over_africa = over_africa
        self.connected_TC = connected_TC
        self.connected_TC_name = connected_TC_name
        self.TC_genesis_time = genesis_time
        self.smooth_lon = smooth_lon
        self.smooth_lat = smooth_lat

        # Also include the new stuff we wnat
        self.CCKW_TB = CCKW_TB
        self.CCKW_VP = CCKW_VP
        self.sig_TB_peak = sig_TB_peak
        self.sig_VP_peak = sig_VP_peak
        self.sig_TB_valley = sig_TB_valley
        self.sig_VP_valley = sig_VP_valley
        self.TB_lat_range = TB_lat_range
        self.VP_lat_range = VP_lat_range
